from ..lstcalendar import LSTCalendar
from . import Observation
from typing import Optional
from perf import track_total_runtime


@track_total_runtime
def is_observable(
    observation: Observation,
    yyyymmdd_start: Optional[str] = None,
    yyyymmdd_end: Optional[str] = None,
    lstCalendar: Optional[LSTCalendar] = None,
    latitude: Optional[str] = None,
    longitude: Optional[str] = None,
) -> bool:
    """
    Determines if an Observation is observable within the specified date and location parameters.

    :param observation: The Observation object to be checked.
    :type observation: Observation
    :param yyyymmdd_start: (Optional) The start date in the format 'YYYYMMDD'.
    :type yyyymmdd_start: Optional[str]
    :param yyyymmdd_end: (Optional) The end date in the format 'YYYYMMDD'. If not provided, the start date is used.
    :type yyyymmdd_end: Optional[str]
    :param lstCalendar: (Optional) An instance of LSTCalendar. If not provided, a new LSTCalendar instance will be created based on the date and location parameters.
    :type lstCalendar: Optional[LSTCalendar]
    :param latitude: (Optional) The latitude for the observation in the format 'D:M:S'. Defaults to -30:42:39.8 (for ASTRAL provider).
    :type latitude: Optional[str]
    :param longitude: (Optional) The longitude for the observation in the format 'D:M:S'. Defaults to 21:26:38.0 (for ASTRAL provider).
    :type longitude: Optional[str]
    :return: True if the observation is observable within the specified parameters, False otherwise.
    :rtype: bool
    """

    # TODO fix me. Some import problem
    from ..lstcalendar import LSTCalendar

    yyyymmdd_end = yyyymmdd_end if yyyymmdd_end else yyyymmdd_start
    calendar = (
        lstCalendar
        if lstCalendar
        else LSTCalendar(yyyymmdd_start, yyyymmdd_end, latitude=latitude, longitude=longitude)
    )

    return bool(observation.observables(calendar))
